/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.app;

import java.awt.*;
import java.io.File;
import java.util.Vector;

import javax.swing.*;
import org.apache.log4j.Logger;

import de.uni_paderborn.fujaba.basic.FileHistory;
import de.uni_paderborn.fujaba.coobra.RecoverDialog;
import de.uni_paderborn.fujaba.preferences.*;
import de.uni_paderborn.fujaba.texteditor.TextEditor;
import de.uni_paderborn.lib.classloader.UPBClassLoader;
import de.upb.lib.plugins.PluginManager;


/**
 * The Fujaba application. Here you can find the main method.
 *
 * @author    $Author: cschneid $
 * @version   $Revision: 1.126 $
 */
public class FujabaApp
{
   /**
    * log4j logging
    */
   private final static transient Logger log = Logger.getLogger (FujabaApp.class);

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private static volatile FujabaApp fujabaApp;

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private static PluginManager pluginManager = KernelInterfaceImpl.get().getManager();


   /**
    * Constructor for class FujabaApp
    */
   private FujabaApp() { }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public static FujabaApp get()
   {
      if (fujabaApp == null)
      {
         fujabaApp = new FujabaApp();
      }

      return fujabaApp;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private static boolean started = false;


   /**
    * Sets the started attribute of the FujabaApp class
    */
   private static void setStarted()
   {
      started = true;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public static FujabaApp start()
   {
      FujabaApp fujabaApp = get();

      if (!started)
      {
         fujabaApp.run();
      }

      return fujabaApp;
   }


   /**
    * this method initializes fujaba. It creates everything the user sees after calling fujaba.
    */
   public void run()
   {
      // create about box as soon as possible
      AboutBox aboutBox = null;

      if (this.getLoadFprFile() == null && this.isShowAboutBox())
      {
         aboutBox = new AboutBox (null, true);
         aboutBox.setCursor (Cursor.getPredefinedCursor (Cursor.WAIT_CURSOR));
         aboutBox.setVisible (true);
      }

      FrameMain frameMain = FrameMain.get();
      frameMain.init();

      try
      {

         frameMain = FrameMain.get();

         if (this.getLoadFprFile() != null)
         {
            // a .fpr file is provided.
            if (log.isDebugEnabled())
            {
               log.debug ("opening " + this.getLoadFprFile());
            }
            File myFile = new File (this.getLoadFprFile());

            if (myFile.exists())
            {
               if (log.isDebugEnabled())
               {
                  log.debug ("loading " + this.getLoadFprFile());
               }
            }

            frameMain.openFile (myFile);
         }
         else
         {
            //plain project was already created in FrameMain.get().init() (s.a.)
         }

         if (aboutBox != null)
         {
            aboutBox.setVisible (false);
         }

         TextEditor.get();

         // show the Fujaba window
         frameMain.setVisible (!isInvisible());

         setStarted();

         // check for backup files from old sessions
         new RecoverDialog (frameMain).check();
      }
      catch (Throwable t)
      {
         if (log.isDebugEnabled())
         {
            log.debug ("uncaught exception: " + t);
         }
         t.printStackTrace();
      }
   }


   // ######################################################################

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public Vector javaFiles = new Vector();

   // ######################################################################

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private String loadFprFile = null;


   /**
    * Sets the loadFprFile attribute of the FujabaApp object
    *
    * @param name  The new loadFprFile value
    */
   public void setLoadFprFile (String name)
   {
      this.loadFprFile = name;
   }


   /**
    * Get the loadFprFile attribute of the FujabaApp object
    *
    * @return   The loadFprFile value
    */
   public String getLoadFprFile()
   {
      return this.loadFprFile;
   }

   // ######################################################################

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private boolean showAboutBox = true;


   /**
    * Sets the showAboutBox attribute of the FujabaApp object
    *
    * @param flag  The new showAboutBox value
    */
   public void setShowAboutBox (boolean flag)
   {
      this.showAboutBox = flag;
   }


   /**
    * Get the showAboutBox attribute of the FujabaApp object
    *
    * @return   The showAboutBox value
    */
   public boolean isShowAboutBox()
   {
      return this.showAboutBox;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private boolean invisible = false;


   /**
    * Sets the invisible attribute of the FujabaApp object
    *
    * @param flag  The new invisible value
    */
   public void setInvisible (boolean flag)
   {
      this.invisible = flag;
   }


   /**
    * Get the invisible attribute of the FujabaApp object
    *
    * @return   The invisible value
    */
   public boolean isInvisible()
   {
      return this.invisible;
   }


   // ######################################################################

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private boolean maximize = false;


   /**
    * Sets the maximize attribute of the FujabaApp object
    *
    * @param maximize  The new maximize value
    */
   public void setMaximize (boolean maximize)
   {
      this.maximize = maximize;
   }


   /**
    * Get the maximize attribute of the FujabaApp object
    *
    * @return   The maximize value
    */
   public boolean isMaximize()
   {
      return maximize;
   }


   /**
    * <b>main</b> function of fujaba. the following parameters are allowed: -noabout : do not
    * show the fujaba about box. -debug : turns on the debug mode. -max : starts fujaba in
    * full screen mode. -start <class> : after initializing start class. -config <dir> : uses
    * <dir>/.fujaba instead of $HOME/.fujaba
    *
    * @param argv  No description provided
    */
   public static void main (String argv[])
   {
      System.out.println ("Welcome to Fujaba");
      Runtime.getRuntime().traceMethodCalls (false);

      FujabaApp app = get();
      boolean parameterError = false;

      for (int i = 0; i < argv.length && !parameterError; i++)
      {
         try
         {
            if (argv[i].equals ("-noabout"))
            {
               app.setShowAboutBox (false);
            }
            else if (argv[i].equals ("-invisible"))
            {
               app.setInvisible (true);
            }
            else if (argv[i].equals ("-last"))
            {
               // open the last project
               File lastFile = FileHistory.get().firstOfHistory();
               System.out.println ("-last loads: " + lastFile.getPath());
               app.setLoadFprFile (lastFile.getPath());
            }
            else if (argv[i].equals ("-debug"))
            {
               DebugPreferences.get().setDebugMode (true);
            }
            else if (argv[i].equals ("-max"))
            {
               app.setMaximize (true);
            }
            else if (argv[i].equals ("-config"))
            {
               PreferencesProperties.setBaseDir (argv[i + 1]);
               System.out.println ("Setting base dir for Fujaba config files to: " + argv[i + 1]);
               i++;
            }
            else if ( ( (argv[i].endsWith (".fpr"))
               ||  ( (argv[i].endsWith (".fpr.gz")))
               ||  (argv[i].endsWith (".xml"))
               ||  (argv[i].endsWith (".xml.gz"))
               )
               &&  (app.getLoadFprFile() == null))
            {
               app.setLoadFprFile (argv[i]);
            }
            else
            {
               System.out.println ("Parameter is unknown:" + argv[i]);
               parameterError = true;
            }
         }
         catch (IndexOutOfBoundsException iex)
         {
            System.out.println ("Too few arguments for: " + argv[i]);
            parameterError = true;
         }
      }
      if (parameterError)
      {
         System.out.println ("The following parameters are valid:");
         System.out.println (" -noabout : do not show the fujaba about box.");
         System.out.println (" -debug : turns on the debug mode.");
         System.out.println (" -max : starts fujaba in full screen mode.");
         System.out.println (" -config <dir> : uses <dir>/.fujaba instead of $HOME/.fujaba");
         System.out.println (" -invisible : makes frame invisible as soon as possible");
      }
      else
      {
         Thread.currentThread().setContextClassLoader (UPBClassLoader.get (UPBClassLoader.DEFAULT_CLASSLOADER));

         // ===== Configure log4j NOW so that framework ready if calls are made to it

         // Turn off log4j default init, not necessary
         System.setProperty ("log4j.defaultInitOverride", "true");

         // Set the property dir as a system property so that it can be used in the log4j config file
         String propertyDir = PreferencesProperties.getPropertyDir();

         String fileSeparator = Character.toString (File.separatorChar);

         // Trim file separator char to match how user.dir etc are stored
         if (propertyDir.endsWith (fileSeparator))
         {
            propertyDir = propertyDir.substring (0, propertyDir.length() - 1);
         }
         System.setProperty ("fujaba.propertydir", propertyDir);

         LoggingPreferences.get(); //.configureLog4J();

         // ===== end of log4j configuration

         // set look and feel
         // todo: store classname instead of laf name as getInstalledLookAndFeels takes a long time
         String lookAndFeel = GeneralPreferences.get().getLookAndFeel();
         String lafClassName = null;
         UIManager.LookAndFeelInfo[] installedLaFs = UIManager.getInstalledLookAndFeels();

         for (int i = 0;  (i < installedLaFs.length) &&  (lafClassName == null); i++)
         {
            if (lookAndFeel.equals (installedLaFs[i].getName()))
            {
               lafClassName = installedLaFs[i].getClassName();
            }
         }
         try
         {
            UIManager.setLookAndFeel (lafClassName);
         }
         catch (Exception e)
         {
         }

         app.run();
      }
   }


   /**
    * @return   the associated PluginManager instance
    */
   public static PluginManager getPluginManager()
   {
      return pluginManager;
   }


   /**
    * sets the pluginManager attribute
    *
    * @param manager
    */
   public static void setPluginManager (PluginManager manager)
   {
      pluginManager = manager;
   }


   /**
    * exit behaviour
    *
    * @return   true if Fujaba calls {@link System#exit} on exit (default)
    */
   public static boolean isAllowedToCallSystemExit()
   {
      return allowedToCallSystemExit;
   }


   /**
    * change the exit behaviour
    *
    * @param allowedToCallSystemExit  true if Fujaba should call {@link System#exit} on exit,
    *      false if not
    */
   public static void setAllowedToCallSystemExit (boolean allowedToCallSystemExit)
   {
      FujabaApp.allowedToCallSystemExit = allowedToCallSystemExit;
   }


   /**
    * true if Fujaba calls {@link System#exit} on exit
    */
   private static boolean allowedToCallSystemExit = true;


   /**
    * Calls System.exit or hides FrameMain
    *
    * @param status  exit code for {@link System#exit}
    * @see           #isAllowedToCallSystemExit
    */
   public static void exit (int status)
   {
      if (isAllowedToCallSystemExit())
      {
         System.exit (status);
      }
      else
      {
         FrameMain.get().dispose();
      }
   }
}

/*
 * $Log: FujabaApp.java,v $
 * Revision 1.126  2005/01/21 12:16:53  cschneid
 * undo/redo move shown immediately, class chooser caches folders too
 *
 */
