/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.app.action;

import java.awt.event.ActionEvent;

import javax.swing.*;

import de.tu_bs.coobra.LocalRepository;
import de.uni_paderborn.fujaba.app.FrameMain;
import de.uni_paderborn.fujaba.coobra.FujabaChangeManager;
import de.uni_paderborn.fujaba.coobra.actions.StartServerRepositoryAction;
import de.uni_paderborn.fujaba.uml.UMLProject;


/**
 * Create a new project.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.18 $
 */
public class NewProjectAction extends ProjectNotSavedAction
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param e  No description provided
    */
   public void actionPerformed (ActionEvent e)
   {
      if (confirmDialog() == JOptionPane.OK_OPTION)
      {
         FrameMain myFrame = FrameMain.get();

         //reset local repsitory when not starting a server
         if (FujabaChangeManager.getVMRepository() != null)
         {
            if (! (e.getSource() instanceof StartServerRepositoryAction))
            {
               myFrame.setUMLProject (null);
               UMLProject.setNewProject (null);
               FujabaChangeManager.setVMRepository (new LocalRepository());
            }
         }

         myFrame.getFrame().setTitle ("Fujaba - From UML To Java And Back Again");

         // for the new project, there will be a new broadcaster
         // so stop every engine of the old one
         // why do you not remove the broadcaster?
         synchronized (myFrame)
         {
            myFrame.setUMLProject (UMLProject.createPlainProject());
            myFrame.closeAllInternalFrames();
            Action action = new ProjectOptionsAction();
            action.actionPerformed (e);
         }
         UMLProject.get().refreshDisplay();

         //disallow undo in new project when not served
         if (! (e.getSource() instanceof StartServerRepositoryAction))
         {
            if (FujabaChangeManager.getVMRepository() != null)
            {
               FujabaChangeManager.getVMRepository().moveChangesIntoChangesBase();
            }
         }
      }
   }
}

/*
 * $Log: NewProjectAction.java,v $
 * Revision 1.18  2004/10/20 17:49:25  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
