/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.basic;

import java.util.Comparator;

import de.uni_paderborn.fujaba.metamodel.FPackage;


/**
 * No comment provided by developer, please add a comment to improve documentation.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.9 $
 */
public class FujabaComparator
{
   // ######################################################################

   /**
    * Get the lessString attribute of the FujabaComparator class
    *
    * @return   The lessString value
    */
   public static Comparator getLessString()
   {
      return
         new Comparator()
         {
            public boolean equals (Object obj)
            {
               return this.compare (this, obj) == 0;
            }


            public int compare (Object o1, Object o2)
            {
               return  ((String) o1).compareTo ((String) o2);
            }
         };
   }

   // ######################################################################

   /**
    * Get the lessClassName attribute of the FujabaComparator class
    *
    * @return   The lessClassName value
    */
   public static Comparator getLessClassName()
   {
      return
         new Comparator()
         {
            public boolean equals (Object obj)
            {
               return this.compare (this, obj) == 0;
            }


            public int compare (Object o1, Object o2)
            {
               return o1.getClass().getName().compareTo (o2.getClass().getName());
            }
         };
   }

   // ######################################################################

   /**
    * Get the lessBasicIncr attribute of the FujabaComparator class
    *
    * @return   The lessBasicIncr value
    */
   public static Comparator getLessBasicIncr()
   {
      return
         new Comparator()
         {
            public boolean equals (Object obj)
            {
               return this.compare (this, obj) == 0;
            }


            public int compare (Object o1, Object o2)
            {
               return  ((BasicIncrement) o1).getID().compareTo ( ((BasicIncrement) o2).getID());
            }
         };
   }

   // ######################################################################

   /**
    * Get the packageComparator attribute of the FujabaComparator class
    *
    * @return   The packageComparator value
    */
   public static Comparator getPackageComparator()
   {
      return
         new Comparator()
         {
            public boolean equals (Object obj)
            {
               return this.compare (this, obj) == 0;
            }


            public int compare (Object o1, Object o2)
            {
               return  ((FPackage) o1).getFullPackageName().compareTo ( ((FPackage) o2).getFullPackageName());
            }
         };
   }
}

/*
 * $Log: FujabaComparator.java,v $
 * Revision 1.9  2004/10/20 17:49:27  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
