/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.coobra.actions;

import java.awt.event.ActionEvent;
import java.math.BigInteger;
import java.util.*;

import javax.swing.*;

import de.tu_bs.coobra.LocalRepository;
import de.tu_bs.coobra.ServerObjectChange;
import de.uni_paderborn.fujaba.app.action.NewProjectAction;
import de.uni_paderborn.fujaba.coobra.FujabaChangeManager;
import de.uni_paderborn.fujaba.uml.UMLProject;


/**
 * Displays a dialog to let the user select a project version to be checked out
 *
 * @author    christian.schneider@uni-kassel.de
 * @version   $Revision: 1.6 $
 */
public class CheckoutVersionAction extends CheckoutAction
{
   /**
    * Perform the action
    *
    * @param event  the ActionEvent for performing the action
    */
   public void actionPerformed (ActionEvent event)
   {
      if (new NewProjectAction().confirmDialog() == JOptionPane.OK_OPTION)
      {
         FujabaChangeManager.setVMRepository (null);
         UMLProject.createPlainProject();
         try
         {
            super.actionPerformed (event);
         }
         finally
         {
            if (FujabaChangeManager.getVMRepository() != null)
            {
               FujabaChangeManager.getVMRepository().setUpdateFilter (null);
            }
         }
      }
   }


   /**
    * dialog for choosing the project version
    */
   CheckoutVersionChooser versionChooser;

   /**
    * version filter
    */
   LocalRepository.UpdateFilter filter =
      new LocalRepository.UpdateFilter()
      {
         /**
          * Called upon update after the changes have been received from the server. The default
          * implementation (that discards nothing) would return the largest of the sequence
          * numbers contained in the changes.
          *
          * @param serverChangesIterator  an iterator w/o remove() through the received changes
          * @return                       the last accepted sequence number from the changes
          */
         public BigInteger filterServerChanges (Iterator serverChangesIterator)
         {
            Map versions = new HashMap();
            List sortedVersions = new LinkedList();
            while (serverChangesIterator.hasNext())
            {
               ServerObjectChange change = (ServerObjectChange) serverChangesIterator.next();
               Object value = versions.get (change.getCause());
               if (value == null || change.getSequenceNumber().compareTo ((BigInteger) value) > 0)
               {
                  versions.put (change.getCause(), change.getServerSequenceNumber());
                  if (value != null && sortedVersions.get (sortedVersions.size() - 1) != change.getCause())
                  {
                     sortedVersions.remove (change.getCause());
                     sortedVersions.add (change.getCause());
                  }
                  else if (value == null)
                  {
                     sortedVersions.add (change.getCause());
                  }
               }
            }
            if (versionChooser == null)
            {
               versionChooser = new CheckoutVersionChooser();
            }
            versionChooser.chooseFromVersions (sortedVersions);
            Object versionCause = versionChooser.getChosenVersion();
            BigInteger sequenceNumber = (BigInteger) versions.get (versionCause);
            if (sequenceNumber == null)
            {
               //rollback all
               sequenceNumber = new BigInteger ("-1");
            }
            return sequenceNumber;
         }
      };


   /**
    * @return   new LocalRepository()
    */
   protected LocalRepository createRepository()
   {
      LocalRepository repository = super.createRepository();
      repository.setUpdateFilter (filter);
      return repository;
   }
}

/*
 * $Log: CheckoutVersionAction.java,v $
 * Revision 1.6  2004/11/25 15:48:34  mtt
 * compiles in jdk 1.4 and jdk 1.5
 *
 */
