/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.fsa.swing.border;

import java.awt.*;

import de.uni_paderborn.fujaba.basic.FontContainer;


/**
 * No comment provided by developer, please add a comment to improve documentation.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.12 $
 */
public class TextBorder extends AbstractColorBorder
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private String text;


   /**
    * Constructor for class TextBorder
    *
    * @param aColor  No description provided
    */
   public TextBorder (Color aColor)
   {
      super (aColor);
   }


   /**
    * Constructor for class TextBorder
    *
    * @param aColor  No description provided
    * @param txt     No description provided
    */
   public TextBorder (Color aColor, String txt)
   {
      this (aColor);
      setText (txt);
   }


   /**
    * Sets the text attribute of the TextBorder object
    *
    * @param txt  The new text value
    */
   public void setText (String txt)
   {
      text = txt;
   }


   /**
    * Get the text attribute of the TextBorder object
    *
    * @return   The text value
    */
   public String getText()
   {
      return text;
   }


   /**
    * Get the borderInsets attribute of the TextBorder object
    *
    * @param c  No description provided
    * @return   The borderInsets value
    */
   public Insets getBorderInsets (Component c)
   {
      return new Insets (13, 0, 0, 0);
   }


   /**
    * Get the borderInsets attribute of the TextBorder object
    *
    * @param c       No description provided
    * @param insets  No description provided
    * @return        The borderInsets value
    */
   public Insets getBorderInsets (Component c, Insets insets)
   {
      insets = getBorderInsets (c);
      return insets;
   }


   /**
    * Get the borderOpaque attribute of the TextBorder object
    *
    * @return   The borderOpaque value
    */
   public boolean isBorderOpaque()
   {
      return false;
   }



   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private Font font = FontContainer.getDefaultFont();


   /**
    * Get the font attribute of the TextBorder object
    *
    * @return   The font value
    */
   public Font getFont()
   {
      return font;
   }


   /**
    * Sets the font attribute of the TextBorder object
    *
    * @param f  The new font value
    */
   public void setFont (Font f)
   {
      font = f;
   }


   /**
    * This method draws the Border for Set objects.
    *
    * @param c       No description provided
    * @param g       No description provided
    * @param x       No description provided
    * @param y       No description provided
    * @param width   No description provided
    * @param height  No description provided
    */
   public void paintBorder (Component c,
                            Graphics g,
                            int x,
                            int y,
                            int width,
                            int height)
   {
      Color oldColor = g.getColor();
      Font oldFont = g.getFont();

      g.setColor (getBorderColor());
      g.setFont (getFont());

      g.drawString (getText(), x, y + 9);

      g.setColor (oldColor);
      g.setFont (oldFont);
   }
}

/*
 * $Log: TextBorder.java,v $
 * Revision 1.12  2004/10/20 17:49:49  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
