/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.gui;

import de.uni_paderborn.fujaba.codegen.AccessorOOHandler;
import de.uni_paderborn.fujaba.codegen.UMLRoleOOHandler;
import de.uni_paderborn.fujaba.metamodel.FRole;
import de.uni_paderborn.fujaba.uml.*;


/**
 * This class is responsible for the completion and implementation of associations. In future
 * this special class will be replaced by the code generation module.
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.30 $
 */
public class AssocCodeGenerator
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int NO_REPAIR_NEEDED = 0;

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int SWITCH_TO_REFERENCE = -2;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int SWITCH_ASSOC = -1;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int STANDARD_ASSOC = 1;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int QUALIFIED_ASSOC = 2 * STANDARD_ASSOC;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int ATTR_QUALIFIER = 2 * QUALIFIED_ASSOC;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int FREE_QUALIFIER = 2 * ATTR_QUALIFIER;

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int LAST_ASSOC_TYPE = 2 * FREE_QUALIFIER;

   // Assoc cardinality.
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int ASSOC_CARD_11 = 2 * LAST_ASSOC_TYPE;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int ASSOC_CARD_1N = 2 * ASSOC_CARD_11;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int ASSOC_CARD_N1 = 2 * ASSOC_CARD_1N;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int ASSOC_CARD_NN = 2 * ASSOC_CARD_N1;

   // Singleton
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private static AssocCodeGenerator theInstance = null;


   /**
    * Constructor for class AssocCodeGenerator
    */
   private AssocCodeGenerator() { }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public static AssocCodeGenerator get()
   {
      if (theInstance == null)
      {
         theInstance = new AssocCodeGenerator();
      }
      return theInstance;
   }


   /**
    * Complete the assoc implementation. param assoc The association which will be completed.
    *
    * @param assoc  No description provided
    */
   public void completeAssoc (UMLAssoc assoc)
   {
      // no access methods for virtual path assocs.
      if (!assoc.isVirtualPath())
      {
         long repairCode = analyzeAssoc (assoc);
         repairAssoc (assoc, repairCode);
      }
   }


   /**
    * Clear the old annotation structure and check for which type of assoc we have to generate
    * code.
    *
    * @param assoc  The assoc to analyze.
    * @return       An error code which identifies the assoc repair action.
    */
   private long analyzeAssoc (UMLAssoc assoc)
   {
      // Get some increments of the assoc.
      UMLRole leftRole = assoc.getLeftRole();
      UMLRole rightRole = assoc.getRightRole();

      if ( (leftRole == null) ||  (rightRole == null))
      {
         // inconsistency found, do not create code
         return NO_REPAIR_NEEDED;
      } // end of if ()

      UMLClass leftTarget = leftRole.getTarget();
      UMLClass rightTarget = rightRole.getTarget();

      if ( (leftTarget == null) ||  (rightTarget == null))
      {
         // inconsistency found, do not create code
         return NO_REPAIR_NEEDED;
      }

      UMLStereotype referenceType =
         UMLStereotypeManager.get().getFromStereotypes (UMLStereotypeManager.REFERENCE);

      if (!leftTarget.hasInStereotypes (referenceType)
         && rightTarget.hasInStereotypes (referenceType)
         && rightRole.getAdornment() != FRole.REFERENCE)
      {
         if (assoc.getDirection() == UMLAssoc.RIGHTLEFT)
         {
            // the assoc must be switched
            return SWITCH_ASSOC;
         } // end of if ()

         // the assoc must be set to reference
         return SWITCH_TO_REFERENCE;
      } // end of if ()

      if (!rightTarget.hasInStereotypes (referenceType)
         && leftTarget.hasInStereotypes (referenceType)
         && leftRole.getAdornment() != FRole.REFERENCE)
      {
         if (assoc.getDirection() == UMLAssoc.LEFTRIGHT)
         {
            // the assoc must be switched
            return SWITCH_ASSOC;
         } // end of if ()

         // the asoc must be set to reference
         return SWITCH_TO_REFERENCE;
      } // end of if ()

      if ( (leftRole.getQualifier() != null) ||  (rightRole.getQualifier() != null))
      {
         UMLQualifier qualifier =
             (leftRole.getQualifier() != null)
            ? leftRole.getQualifier()
            : rightRole.getQualifier();
         int card =
             (leftRole.getQualifier() != null)
            ? getAssocCard (leftRole, rightRole)
            : getAssocCard (rightRole, leftRole);

         if (qualifier.getFQualifiedAttr() != null)
         {
            // This is a qualified assoc whith an attribute as qualifier.
            return QUALIFIED_ASSOC | ATTR_QUALIFIER | card;
         }
         else
         {
            // This is a qualfied assoc with a free key as qualifier.
            return QUALIFIED_ASSOC | FREE_QUALIFIER | card;
         }
      }

      // If this point is reached we assume it's a standard assoc.
      return STANDARD_ASSOC | getAssocCard (leftRole, rightRole);
   } // analyzeAssoc


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param assoc       No description provided
    * @param repairCode  No description provided
    */
   protected void repairAssoc (UMLAssoc assoc, long repairCode)
   {
      if (repairCode == SWITCH_ASSOC)
      {
         UMLRole leftRole = assoc.getLeftRole();
         UMLRole rightRole = assoc.getRightRole();
         assoc.setLeftRole (rightRole);
         assoc.setRightRole (leftRole);
      }

      else if (repairCode == SWITCH_TO_REFERENCE)
      {
         UMLRole leftRole = assoc.getLeftRole();
         UMLRole rightRole = assoc.getRightRole();
         if (assoc.getDirection() == UMLAssoc.LEFTRIGHT)
         {
            rightRole.setAdornment (FRole.REFERENCE);
         } // end of if ()

         else
         {
            leftRole.setAdornment (FRole.REFERENCE);
         }
      }
      else
      {
         if ( (repairCode & STANDARD_ASSOC) != 0)
         {
            repairStandardAssoc (assoc);
         }
         else if ( (repairCode & QUALIFIED_ASSOC) != 0)
         {
            if ( (repairCode & ATTR_QUALIFIER) != 0)
            {
               // Attribute qualifier.
               repairQualifiedAttrAssoc (assoc, repairCode);
            }
            else
            {
               // Free key qualifier.
               repairQualifiedFreeAssoc (assoc, repairCode);
            }
         }
      }
   } // repairIncrement


   /**
    * Get the templateBlockName attribute of the AssocCodeGenerator class
    *
    * @param blockName  No description provided
    * @param cls        No description provided
    * @param attr       No description provided
    * @return           The templateBlockName value
    * @deprecated       Use UMLClassOOHandler.getTemplateBlockName instead
    */
   public static String getTemplateBlockName (String blockName, UMLClass cls, UMLAttr attr)
   {
      return AccessorOOHandler.getTemplateBlockName (blockName, cls, attr);
   }


   /**
    * Access method for an one to n association.
    *
    * @param listener  The object added.
    * @deprecated      Use UMLClassOOHandler.addToTemplateBlockNameListeners instead
    */
   public static void addToTemplateBlockNameListeners (TemplateBlockNameListener listener)
   {
      AccessorOOHandler.addToTemplateBlockNameListeners (listener);
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param listener  No description provided
    * @deprecated      Use UMLClassOOHandler.removeFromTemplateBlockNameListeners instead
    */
   public static void removeFromTemplateBlockNameListeners (TemplateBlockNameListener listener)
   {
      AccessorOOHandler.removeFromTemplateBlockNameListeners (listener);
   }


   /**
    * Generates new code for a qualified association. The qualifier is an attribute of the
    * class.
    *
    * @param repairCode  Flag for additional information on the assoc.
    * @param assoc       No description provided
    */
   protected void repairQualifiedAttrAssoc (UMLAssoc assoc, long repairCode)
   {
      // Destroy the annotation structure, the access methods and the assoc attribute.
      assoc.removeAttrs();
   }


   /**
    * Generates new code for a qualified association. The qualifier was specified separately
    * and is not an attribute of the class.
    *
    * @param repairCode  Flag for additional information on the assoc.
    * @param assoc       No description provided
    */
   protected void repairQualifiedFreeAssoc (UMLAssoc assoc, long repairCode)
   {
      // Destroy the annotation structure, the access methods and the assoc attribute.
      assoc.removeAttrs();
   } // repairQualifiedFreeAssoc


   /**
    * Repairs an assoc without qualifiers.
    *
    * @param assoc  The association to repair.
    */
   protected void repairStandardAssoc (UMLAssoc assoc)
   {
      // Get all needed increments.
      UMLRole leftRole = assoc.getLeftRole();
      UMLRole rightRole = assoc.getRightRole();

      UMLClass leftTarget = leftRole.getTarget();
      UMLClass rightTarget = rightRole.getTarget();

      // Destroy the annotation structure, the access methods and the assoc attribute.
      assoc.removeAttrs();

      // ------------------------------------------------------------------
      // If the roles have no cardinality, assume it's a To One cardinality.
      // ------------------------------------------------------------------

      // Create new attribute and new access methods for the roles.
      if (rightTarget != null
         && ! (rightRole.getAdornment() == FRole.REFERENCE))
      {
         if (leftRole.getCard() == null)
         {
            // Create a new cardinality.
            UMLCardinality newElem = new UMLCardinality ("1", null);
            newElem.setGenerated (false);
            leftRole.setCard (newElem);
         }
      } // end of if ()

      // Create new attribute and new access methods for the roles.
      if (leftTarget != null
         && ! (leftRole.getAdornment() == FRole.REFERENCE))
      {
         if (rightRole.getCard() == null)
         {
            UMLCardinality newElem = new UMLCardinality ("1", null);
            newElem.setGenerated (false);
            rightRole.setCard (newElem);
         }
      }
   } // repairStandardAssoc


   /**
    * Returns the type of the assoc.
    *
    * @param leftRole   Left role of the association.
    * @param rightRole  The associated partner role.
    * @return           The assocCard value
    */
   private int getAssocCard (UMLRole leftRole, UMLRole rightRole)
   {
      int assocCard = ASSOC_CARD_11;

      if ( (leftRole.getCard().getUpperBound() == 1)
         &&  (rightRole.getCard().getUpperBound() == 1))
      {
         // One To One Assoc.
         assocCard = ASSOC_CARD_11;
      }
      else if (leftRole.getCard().getUpperBound() == 1)
      {
         // One To Many Assoc.
         assocCard = ASSOC_CARD_1N;
      }
      else if (rightRole.getCard().getUpperBound() == 1)
      {
         // Many To One Assoc.
         assocCard = ASSOC_CARD_N1;
      }
      else
      {
         assocCard = ASSOC_CARD_NN;
      }

      return assocCard;
   } // getAssocCard


   /**
    * Get the referenceTemplateName attribute of the CodeGenFactory object
    *
    * @return       The referenceTemplateName value
    * @deprecated   Use UMLClassOOHandler.getReferenceTemplateName instead
    */
   public static String getReferenceTemplateName()
   {
      return UMLRoleOOHandler.getReferenceTemplateName();
   }


   /**
    * Get the assocTemplateName attribute of the CodeGenFactory object
    *
    * @return       The assocTemplateName value
    * @deprecated   Use UMLClassOOHandler.getAssocTemplateName instead
    */
   public static String getAssocTemplateName()
   {
      return UMLRoleOOHandler.getAssocTemplateName();
   }

}

/*
 * $Log: AssocCodeGenerator.java,v $
 * Revision 1.30  2004/11/13 12:04:15  lowende
 * Removed some compile warnings.
 *
 */
