/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.packagediagrams.actions;

import java.awt.event.ActionEvent;
import java.util.Iterator;

import javax.swing.*;

import de.uni_paderborn.fujaba.app.FrameMain;
import de.uni_paderborn.fujaba.packagediagrams.DiagramUsage;
import de.uni_paderborn.fujaba.packagediagrams.PackageDiagram;
import de.uni_paderborn.fujaba.uml.*;
import de.uni_paderborn.fujaba.usecase.UsecaseDiagram;


/**
 * No comment provided by developer, please add a comment to improve documentation.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.14 $
 */
public class GoToDiagramAction extends AbstractAction
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param event  No description provided
    */
   public void actionPerformed (ActionEvent event)
   {
      Object source = event.getSource();
      if (source instanceof Iterator)
      {
         Iterator iter = (Iterator) source;
         if (iter.hasNext())
         {
            source = iter.next();
         }
      }

      if (source instanceof DiagramUsage)
      {
         try
         {
            DiagramUsage usage = (DiagramUsage) source;
            UMLDiagram umlDiagram = usage.getDiagram();
            if (umlDiagram == null)
            {
               if (0 == usage.getDiagramKind())
               {
                  umlDiagram = new PackageDiagram (usage.getName(), UMLProject.get());
               }
               else if (1 == usage.getDiagramKind())
               {
                  umlDiagram = new UsecaseDiagram (usage.getName(), UMLProject.get());
               }
               else if (2 == usage.getDiagramKind())
               {
                  umlDiagram = new UMLClassDiagram (usage.getName(), UMLProject.get());
               }
               else if (3 == usage.getDiagramKind())
               {
                  umlDiagram = new UMLActivityDiagram (usage.getName(), UMLProject.get());
               }

               usage.setDiagram (umlDiagram); //Makes a connection between usage and umlDiagram

               if (umlDiagram instanceof UsecaseDiagram)
               {
                   ((UsecaseDiagram) umlDiagram).init();
               }
               // Updates the project tree.
               FrameMain.get().createNewTreeItems();
            }
            // Select the item in the project tree.
            FrameMain.get().selectTreeItem (umlDiagram);
            UMLProject.get().refreshDisplay();
         }
         finally
         {
         }
      }
   } // actionPerformed

}

/*
 * $Log: GoToDiagramAction.java,v $
 * Revision 1.14  2004/10/20 17:50:02  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
