/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.uml;

import java.util.StringTokenizer;

import de.uni_paderborn.fujaba.metamodel.*;
import de.uni_paderborn.fujaba.uml.unparse.UMLUnparseGetter;


/**
 * Provides commentary nodes for the diags.
 *
 * <h2>Associations</h2>
 *
 * <pre>
 *                1                       1
 * UMLCommentary --------------------------- UMLIncrement
 *                comment        revComment
 * </pre>
 *
 * @author    $Author: cschneid $
 * @version   $Revision: 1.53 $
 */
public class UMLCommentary extends UMLIncrement implements FCommentary
{

   /**
    * Constructor for class UMLCommentary
    */
   public UMLCommentary()
   {
      super();
   }


   /**
    * Constructor for class UMLCommentary
    *
    * @param coobraPersistent  No description provided
    */
   public UMLCommentary (boolean coobraPersistent)
   {
      super (coobraPersistent);
   }


   /**
    * Constructor for class UMLCommentary
    *
    * @param text  No description provided
    */
   public UMLCommentary (String text)
   {
      super();
      setText (text);
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   protected String createUnparseModuleName()
   {
      return UMLUnparseGetter.getUnparseModuleName (this);
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param value  No description provided
    */
   public void swapVisibilityTo (boolean value) { }


   /**
    * Get the visibilityOfFirstIncr attribute of the UMLCommentary object
    *
    * @return   The visibilityOfFirstIncr value
    */
   public boolean getVisibilityOfFirstIncr()
   {
      return false;
   }

   // ######################################################################

   /**
    * <pre>
    *                1                       1
    * UMLCommentary --------------------------- UMLIncrement
    *                comment        revComment
    * </pre>
    */
   private transient UMLIncrement revComment; // reverse attribute comment

   // TODO-BEGIN: Merge with JDK 1.5
   /**
    * Get the revComment attribute of the UMLCommentary object
    *
    * @return   The revComment value
    */
   public UMLIncrement getRevComment()
   {
      return revComment;
   } // getRevComment


   /*
    *  (non-Javadoc)
    *  @see de.uni_paderborn.fujaba.metamodel.FCommentary#getFRevComment()
    */
   /**
    * Get the fRevComment attribute of the UMLCommentary object
    *
    * @return   The fRevComment value
    */
   public FIncrement getFRevComment()
   {
      return getRevComment();
   }
   // TODO-END

   /**
    * Sets the revComment attribute of the UMLCommentary object
    *
    * @param revComment  The new revComment value
    */
   public void setRevComment (FIncrement revComment)
   {
      if (this.revComment != revComment)
      {
         // newPartner
         UMLIncrement oldRevComment = this.revComment;
         if (this.revComment != null)
         {
            // inform old partner
            this.revComment = null;

            oldRevComment.setComment (null);
         } // if

         this.revComment = (UMLIncrement) revComment;
         if (revComment != null)
         {
            // inform new partner
            revComment.setComment (this);
         } // if

         firePropertyChange (REV_COMMENT_PROPERTY, oldRevComment, revComment);
      } // if

   } // setRevComment

   // ######################################################################

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param text  No description provided
    * @return      No description provided
    */
   public String deleteBlancLines (String text)
   {
      StringTokenizer st = new StringTokenizer (text, "\n", true);
      boolean blancLines = true;
      String str;
      StringBuffer initialBlancs = new StringBuffer();

      // get the blancs in front of the first non-blanc line
      while (blancLines && st.hasMoreTokens())
      {
         str = st.nextToken();
         if (str.trim().length() > 0)
         {
            blancLines = false;
            StringBuffer strBuf = new StringBuffer (str);
            boolean ready = false;
            int i = 0;
            while (!ready && i < strBuf.length())
            {
               char chr = strBuf.charAt (i);
               if (chr == ' ' || chr == '\t')
               {
                  initialBlancs.append (chr);
               }
               else
               {
                  ready = true;
               }
               i++;
            }
         }
      }
      return  (text.trim().length() > 0) ? initialBlancs + text.trim() + "\n" : "";
   }


   /**
    * Get the incrText attribute of the UMLCommentary object
    *
    * @return   The incrText value
    */
   public String getIncrText()
   {
      return getText();
   }


   /**
    * UMLAttribute: '+ text : String'
    */
   private String text = "";


   /**
    * Get the text attribute of the UMLCommentary object
    *
    * @return   The text value
    */
   public String getText()
   {
      return text;
   } // getText


   /**
    * Get the commentedText attribute of the UMLCommentary object
    *
    * @return   The commentedText value
    */
   public String getCommentedText()
   {
      if (!this.text.trim().startsWith ("/*") && !this.text.trim().startsWith ("//"))
      {
         return  ("/*\n" + this.text + "*/\n");
      }
      else
      {
         return text;
      }
   }


   /**
    * Sets the text attribute of the UMLCommentary object
    *
    * @param text  The new text value
    * @return      No description provided
    */
   public String setText (String text)
   {
      if (text != null && !this.text.equals (text))
      {
         String oldValue = this.text;
         this.text = deleteBlancLines (text);
         firePropertyChange ("text", oldValue, this.text);
      } // if

      return this.text;
   } // setText


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public void removeYou()
   {
      setRevComment (null);

      super.removeYou();
   }


   /**
    * Query the logical parent of this element (e.g. package of a class, diagram of an object).
    *
    * @return   the logical parent of this element, may not return null unless this is the top level node (project)
    */
   public FElement getParentElement()
   {
      return getRevComment();
   }
}

/*
 * $Log: UMLCommentary.java,v $
 * Revision 1.53  2005/02/14 14:28:19  cschneid
 * introduced message view, getParentElement() for FElement, code generation and compiler messages in message view
 *
 */
