/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase;

import de.uni_paderborn.fujaba.metamodel.FElement;

import de.uni_paderborn.fujaba.uml.UMLDiagram;
import de.uni_paderborn.fujaba.uml.UMLProject;


/**
 * A class to implement the rule, that all {@link Usecase}s have to be inside the System-box,
 * see {@link UsecaseSystem}.
 *
 * @author    $Author: cschneid $
 * @version   $Revision: 1.15 $
 * @see       Usecase
 * @see       UsecaseSystem
 */
public class UsecaseSystemDiagram extends UMLDiagram
{
   /**
    * Constructor for class UsecaseSystemDiagram
    */
   public UsecaseSystemDiagram()
   {
      super();
   }


   /**
    * Constructor for class UsecaseSystemDiagram
    *
    * @param name     No description provided
    * @param project  No description provided
    */
   public UsecaseSystemDiagram (String name, UMLProject project)
   {
      super (name, project);
   }


   /**
    * UMLMethod: '+ removeYou () : Void'
    */
   public void removeYou()
   {
      UsecaseSystem tmpSystem = getSystem();
      if (tmpSystem != null)
      {
         setSystem (null);
      } // if

   }


   /**
    * Query the logical parent of this element (e.g. package of a class, diagram of an object).
    *
    * @return   the logical parent of this element;
    */
   public FElement getParentElement()
   {
      return getSystem();
   }


   /**
    * <pre>
    *                1         assoc         1
    * UsecaseSystem --------------------------- UsecaseSystemDiagram
    *                system      systemDiagram
    * </pre>
    */

   private UsecaseSystem system;


   /**
    * UMLMethod: '+ setSystem (value : UsecaseSystem) : Boolean'
    *
    * @param value  The new system value
    * @return       No description provided
    */
   public boolean setSystem (UsecaseSystem value)
   {
      boolean changed = false;
      if (this.system != value)
      {
         if (this.system != null)
         {
            UsecaseSystem oldValue = this.system;
            this.system = null;
            oldValue.setSystemDiagram (null);
         }
         this.system = value;
         if (value != null)
         {
            value.setSystemDiagram (this);
         }
         changed = true;
      }
      return changed;
   }


   /**
    * UMLMethod: '+ getSystem () : UsecaseSystem'
    *
    * @return   The system value
    */
   public UsecaseSystem getSystem()
   {
      return this.system;
   }
}

/*
 * $Log: UsecaseSystemDiagram.java,v $
 * Revision 1.15  2005/02/14 14:28:23  cschneid
 * introduced message view, getParentElement() for FElement, code generation and compiler messages in message view
 *
 */
