/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase.unparse;

import javax.swing.*;

import de.uni_paderborn.fujaba.fsa.*;
import de.uni_paderborn.fujaba.fsa.listener.*;
import de.uni_paderborn.fujaba.fsa.swing.*;
import de.uni_paderborn.fujaba.fsa.unparse.AbstractUnparseModule;
import de.uni_paderborn.fujaba.fsa.unparse.LogicUnparseInterface;
import de.uni_paderborn.fujaba.usecase.*;


/**
 * Class for displaying a {@link UsecaseActor} on screen. A UsecaseActor is the funny little
 * stickman in Use Case Diagrams. With arms up, of course.
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.17 $
 * @see       UsecaseDiagram
 * @see       UsecaseActor
 * @see       Usecase
 * @see       UsecaseUsage
 */
public class UMUsecaseActor extends AbstractUnparseModule
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int LAYER = JLayeredPane.DEFAULT_LAYER.intValue() + 20;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int DRAG_LAYER = LAYER + 1;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static String LAYER_NAME = "UseCaseActorLayer";

   static
   {
      LayerManager.get().addToLayerPositions (LAYER_NAME, new Integer (LAYER));
      LayerManager.get().addToLayerPositions (LayerManager.DRAG_LAYER_PREFIX + LAYER_NAME, new Integer (DRAG_LAYER));
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param parent  No description provided
    * @param incr    No description provided
    * @return        No description provided
    */
   public FSAObject create (FSAObject parent, LogicUnparseInterface incr)
   {
      FSAContainer container = (FSAContainer) parent;
      UsecaseActor actor = (UsecaseActor) incr;

      // the main panel for the UsecaseActor
      FSAPanel panel = new FSAPanel (incr, getMainFsaName(), parent.getJComponent());
      panel.setLayout (new ColumnRowLayout (2, ColumnRowLayout.COLUMN));
      panel.setOpaque (false);

      // place in a layer relative to other Use Case Diagram components
      LayerManager.setLayerType (panel.getJComponent(), LAYER_NAME);

      // the stickman
      JStickMan stickman = new JStickMan();
      stickman.setOpaque (false);

      // the actor's name, centered
      FSATextFieldLabel actorNameTextField
          = new FSATextFieldLabel (actor, "name", panel.getJComponent());
      actorNameTextField.setHorizontalAlignment (SwingConstants.CENTER);
      actorNameTextField.setOpaque (false);

      // default updater handles text changes
      actorNameTextField.addToUpdater
          (actorNameTextField.createDefaultUpdater());

      panel.addToComponents (stickman);
      panel.addToChildren (actorNameTextField);
      container.addToChildren (panel);

      // add mouse listener for moving and selecting
      JComponent jPanel = panel.getJComponent();
      AscendDescendMouseHandler.addMouseInputListener (jPanel, SelectionMouseListener.get());
      AscendDescendMouseHandler.addMouseInputListener (jPanel, DragMouseListener.get());
      SelectionListenerHelper.addSelectionListener (jPanel, BorderHighlighter.get());

      return panel;
   }
}

/*
 * $Log: UMUsecaseActor.java,v $
 * Revision 1.17  2004/10/22 16:41:44  lowende
 * Deprecated warnings removed. Other compile warnings removed.
 *
 */
